@@ -1,9 +1,9 @@
 #' Summary plots for cat2cat results
 #' @description This function help to understand properties of cat2cat results.
 #' It is recommended to run it before further processing, like next iterations.
-#' @param data data.frame - one of the data.frames returned by the `cat2cat` function.
-#' @param weis character - name of a certain wei_*_c2c column, added by cat2cat function. Default wei_freq_c2c
-#' @param type character - one of 3 types `"both"`, `"hist"`, `"bar"`.
+#' @param data `data.frame` - one of the data.frames returned by the `cat2cat` function.
+#' @param weis `character(1)` - name of a certain wei_*_c2c column, added by cat2cat function. Default `wei_freq_c2c`.
+#' @param type `character(1)` - one of 3 types `"both"`, `"hist"`, `"bar"`.
 #' @return base plot graphics
 #' @importFrom graphics hist barplot par
 #' @importFrom utils head
@@ -24,7 +24,8 @@
 #' plot_c2c(occup_2$old, type = c("bar"))
 plot_c2c <- function(data, weis = "wei_freq_c2c", type = c("both", "hist", "bar")) {
   stopifnot(is.data.frame(data))
-  stopifnot("There are no cat2cat additional wei_* columns/variables" = !is.null(data[[weis]]))
+  stopifnot(is.character(weis) && (length(weis) == 1))
+  stopifnot("There are no cat2cat additional wei_* columns/variables" = weis %in% colnames(data))
   stopifnot("There are no replications. Probably you should switch between old/new period." = nrow(data) != sum(data[[weis]]))
   type <- match.arg(type)
 
